#include "jabber.h"

conn *dispatch(char *p, conn *c, int flag)
{
	conn *c_new = NULL;
	xpt *xml, *next;
	int tty;
	char *buff;

	DBUG("DISPATCH",p);

	switch(flag)
	{
		case IO_REMOVE:
			DBUG("Removing connection at handler level for ",c->name)
			etc.cli->packets = packet_add(etc.cli->packets,"\nERROR: Server Disconnect!\njabber>",NULL);
			break;

		case IO_INIT:
			DBUG("Initialization from IO loop","")
			tty = open("/dev/tty",O_RDWR | O_NONBLOCK);
			c_new = conn_add(NULL,tty);
			c_new->type = CONN_NORMAL;
			c_new->name = strdup("/dev/tty");
			c_new->packets = packet_add(NULL,"jabber>",NULL);
			etc.cli = c_new;
			break;

		case IO_NORMAL:
			if(c == etc.server)
			{
				DBUG("Processing from server","");
				if(c->xp == NULL) /* initialize xp if it hasn't been yet */
					c->xp = xpt_packet_new();
				xpt_packet_data(c->xp,p,strlen(p));

				/* check if this is a new unseen connection, we set c->ver after we've seen it already */
				DBG_VAL("c->xp->name",c->xp->name);
				DBG_VAL("c->ver",c->ver);
				if(c->xp->name != NULL && strneq(c->ver,"client"))
				{
					if(strcmp(c->xp->name,"jabber") != 0)
					{
						DBUG("Invalid XML header sent!","");
						return;
					}else{
						/* we're ok with this server so far, flag it thus */
						if(c->ver != NULL)
							free(c->ver);
						c->ver = strdup("client");
						buff = strgrow(NULL,"Connected.\n",0,0);
						if(c->name != NULL)
						{
							buff = strgrow(buff,"Server: ",1,0);
							buff = strgrow(buff,c->name,1,0);
							buff = strgrow(buff,"\n",1,0);
						}
						if(jpair_get(c->xp->attributes, "version") != NULL)
						{
							buff = strgrow(buff,"Version: ",1,0);
							buff = strgrow(buff,jpair_getval(c->xp->attributes, "version"),1,0);
							buff = strgrow(buff,"\n",1,0);
						}
						if(jpair_get(c->xp->attributes, "protocol") != NULL)
						{
							buff = strgrow(buff,"Protocol: ",1,0);
							buff = strgrow(buff,jpair_getval(c->xp->attributes, "protocol"),1,0);
							buff = strgrow(buff,"\n",1,0);
						}
						buff = strgrow(buff,"jabber>",1,0);
						etc.cli->packets = packet_add(etc.cli->packets,buff,NULL);
						free(buff);
						etc.mode = MODE_NORMAL;
					}
				}

				next = xml = c->xp->packets;
				while(next != NULL)
				{
					xml = next;
					next = next->next;
					xml->next = NULL; /* single it out */
					handle_packet(c, xml);
				}
				free_xpt(c->xp->packets);
				c->xp->packets = NULL;
				if(c->xp->flag != XPT_POOLFLAG_NONE)
				{
					switch(c->xp->flag)
					{
						case XPT_POOLFLAG_ERROR:
							DBUG("XML parsing error!",c->name);
							break;
						case XPT_POOLFLAG_CLOSE:
							DBUG("Recieved XML close",c->name);
							break;
					}
					/*handle_logout(c,NULL);*/
					(int)c_new = -1;
				}
			}else{
				DBUG("Processing from server","");
				c_new = handle_command(p,c);
			}
			break;
	}

	return c_new;
}

void handle_packet(conn *c, xpt *x)
{
	char *type, *buff;
	char intbuff[10] = "";
	jpair *p;
	xpt *next;

	type = x->name;
	p = xpt2jpair(x->children);
	if(type == NULL || p == NULL) /* not a valid incoming packet */
	{
		DBG_MSG("Illegal Packet Received");
		return;
	}

	DBG_VAL("normal_packet",type);

	if(strcmp(type,"message") == 0)
	{
		if(jpair_get(p, "priority") != NULL)
			sprintf(intbuff,"%d",jpair_getval(p, "priority"));
		buff = strgrow(NULL,"\n===== Incoming Message ",0,0);
		if(jpair_get(x->attributes, "type"))
		{
			buff = strgrow(buff,"[",1,0);
			buff = strgrow(buff,jpair_getval(x->attributes, "type"),1,0);
			buff = strgrow(buff,"] ",1,0);
		}
		buff = strgrow(buff,"=====\n",1,0);
		buff = strgrow(buff,"From: ",1,0);
		buff = strgrow(buff,jpair_getval(p, "from"),1,0);
		if(jpair_get(p, "thread") != NULL)
		{
			buff = strgrow(buff,"\nThread: ",1,0);
			buff = strgrow(buff,jpair_getval(p, "thread"),1,0);
		}
		if(jpair_get(p, "priority") != NULL)
		{
			buff = strgrow(buff,"\nPriority: ",1,0);
			buff = strgrow(buff,intbuff,1,0);
		}
		if(jpair_get(p, "subject") != NULL)
		{
			buff = strgrow(buff,"\nSubject: ",1,0);
			buff = strgrow(buff,jpair_getval(p, "subject"),1,0);
		}
		buff = strgrow(buff,"\n\n",1,0);
		buff = strgrow(buff,jpair_getval(p, "say"),1,0);
		buff = strgrow(buff,"\n======= End  Message =======\njabber>",1,0);
		etc.cli->packets = packet_add(etc.cli->packets,buff,NULL);
		free(buff);
	}
	if(strcmp(type,"status") == 0)
	{
		if(jpair_get(p, "priority") != NULL)
			sprintf(intbuff,"%d",jpair_getval(p, "priority"));
		buff = strgrow(NULL,"\n===== Incoming Status ",0,0);
		if(jpair_get(x->attributes, "type"))
		{
			buff = strgrow(buff,"[",1,0);
			buff = strgrow(buff,jpair_getval(x->attributes, "type"),1,0);
			buff = strgrow(buff,"] ",1,0);
		}
		buff = strgrow(buff,"=====\n",1,0);
		buff = strgrow(buff,"From: ",1,0);
		buff = strgrow(buff,jpair_getval(p, "from"),1,0);
		if(jpair_get(p, "priority") != NULL)
		{
			buff = strgrow(buff,"\nPriority: ",1,0);
			buff = strgrow(buff,intbuff,1,0);
		}
		if(jpair_get(p, "icon") != NULL)
		{
			buff = strgrow(buff,"\nIcon: ",1,0);
			buff = strgrow(buff,jpair_getval(p, "icon"),1,0);
		}
		buff = strgrow(buff,"\n\n",1,0);
		buff = strgrow(buff,jpair_getval(p, "say"),1,0);
		buff = strgrow(buff,"\n======= End  Status =======\njabber>",1,0);
		etc.cli->packets = packet_add(etc.cli->packets,buff,NULL);
		free(buff);
	}
	if(strcmp(type,"roster") == 0)
	{
		buff = strgrow(NULL,"\n===== Incoming Roster ======\n",0,0);
		next = x->children;
		while(next != NULL)
		{
			if(next->type != XPT_TAG)
				continue;

			buff = strgrow(buff,"(",1,0);
			buff = strgrow(buff,jpair_getval(next->attributes,"name"),1,0);
			buff = strgrow(buff,") ",1,0);
			buff = strgrow(buff,xpt2str_chain(next->children),1,1);
			buff = strgrow(buff,"\n",1,0);
			next = next->next;
		}
		buff = strgrow(buff,"======= End Roster =======\njabber>",1,0);
		etc.cli->packets = packet_add(etc.cli->packets,buff,NULL);
		free(buff);
	}
	free_jpair(p);
}

conn *handle_command(char *line, conn *c)
{
	char *arg;
	static char *lastarg = NULL;
	static char *buff = NULL;
	int s, flag;
	conn *c_new;

	DBUG("Handle command",line)


	switch(etc.mode)
	{
		case MODE_NORMAL:
			line = strrepl(line,"\r"," ");
			line = strrepl(line,"\n"," ");
			line = strrepl(line,"\t"," ");
			arg = strstr(line," ");
			if(arg != NULL)
			{
				arg[0] = '\0';
				++arg;
				arg = strrepl(arg," ","");
			}/* now we have a pointer to the command and to the argument */


			flag = jpair_getflag(etc.cmd,line);
			if(flag == CMD_LOGIN || flag == CMD_LOGOUT || flag == CMD_MESSAGE || flag == CMD_STATUS || flag == CMD_WHO || flag == CMD_ADD)
			{
				if(etc.server == NULL)
				{
					c->packets = packet_add(c->packets,"Error, not connected to a server!\n",NULL);
					break;
				}
			}
			switch(flag)
			{
				case CMD_CONNECT:
					if(arg == NULL || strlen(arg) <= 0)
					{
						c->packets = packet_add(c->packets,"Please enter a server name to connect to: connect server.com\n",NULL);
						break;
					}
					s = make_netsocket(5222,arg,1);
					if(s < 0)
					{
						c->packets = packet_add(c->packets,"Couldn't connect to server!\n",NULL);
						break;
					}
					etc.mode = MODE_CONNECT;
					c_new = conn_add(NULL,s);
					c_new->name = strdup(arg);
					etc.server = c_new;
					deliver_connection();
					return c_new;
					break;
				case CMD_LOGIN:
					if(arg == NULL || strlen(arg) <= 0)
					{
						c->packets = packet_add(c->packets,"Please enter a user id: login myuserid\n",NULL);
						break;
					}
					lastarg = strdup(arg);
					etc.mode = MODE_LOGIN;
					c->packets = packet_add(c->packets,"Password:",NULL);
					break;
				case CMD_LOGOUT:
					break;
				case CMD_MESSAGE:
					if(arg == NULL || strlen(arg) <= 0)
					{
						c->packets = packet_add(c->packets,"Please enter a user id to send to: message recipient\n",NULL);
						break;
					}
					lastarg = strdup(arg);
					etc.mode = MODE_MESSAGE;
					c->packets = packet_add(c->packets,"Type your message, ending it with a period '.' by itself on a line:\n",NULL);
					break;
				case CMD_STATUS:
					if(arg != NULL)
						lastarg = strdup(arg);
					else
						lastarg = NULL;
					etc.mode = MODE_STATUS;
					c->packets = packet_add(c->packets,"Type your status description, ending it with a period '.' by itself on a line:\n",NULL);
					break;
				case CMD_WHO:
					deliver_who();
					break;
				case CMD_ADD:
					if(arg == NULL || strlen(arg) <= 0)
					{
						c->packets = packet_add(c->packets,"Please enter a user id to add: add userid\n",NULL);
						break;
					}
					deliver_add(arg);
					break;
				case CMD_HELP:
					c->packets = packet_add(c->packets,"Commands:\n\tconnect server.com\n\tlogin username\n\tmessage userid@server.com\n\tclear\n\tstatus [online|offline]\n\twho\n\tadd\n\thelp (this command)\n",NULL);
					break;
				case CMD_EXIT:
					raise(SIGTERM);
					break;
				case CMD_CLEAR:
					printf("%c[2J\n", '\033');
					break;
				default:
					printf("Recieved UNKNOWN Command\n");
					break;
			}
			break;
		case MODE_CONNECT:
			c->packets = packet_add(c->packets,"Please wait, attempting to connect to server...\n",NULL);
			break;
		case MODE_WHO:
			break;
		case MODE_LOGIN:
			line = strrepl(line,"\r","");
			line = strrepl(line,"\n","");
			if(buff == NULL)
			{
				buff = strdup(line);
				c->packets = packet_add(c->packets,"Nickname:",NULL);
			}else{
				deliver_login(lastarg,buff,line);
				free(lastarg);
				free(buff);
				buff = lastarg = NULL;
				etc.mode = MODE_NORMAL;
			}
			break;
		case MODE_MESSAGE:
			if(strcspn(line, " \t\r\n") == 1 && line[0] == '.')
			{
				deliver_message(lastarg, NULL, NULL, buff, NULL, NULL, 0);
				free(buff);
				free(lastarg);
				buff = lastarg = NULL;
				etc.mode = MODE_NORMAL;
			}else{
				buff = strgrow(buff,line,1,0);
			}
			break;
		case MODE_STATUS:
			if(strcspn(line, " \t\r\n") == 1 && line[0] == '.')
			{
				deliver_status(lastarg, buff);
				free(buff);
				free(lastarg);
				buff = lastarg = NULL;
				etc.mode = MODE_NORMAL;
			}else{
				buff = strgrow(buff,line,1,0);
			}
			break;
		default:
			DBG_MSG("In UNKNOWN mode!");
			etc.mode = MODE_NORMAL;
			break;
	}

	if(etc.mode == MODE_NORMAL)
		c->packets = packet_add(c->packets,"jabber>",NULL);

	return NULL;
}


