#include "etherx.h"

/*
 * Transports is a linked list with one entry for every connected transport
 * Lookups is a linked list with one entry for every NAME for the transports(it's larger than transports)
 */

void lookup_add(char *name, transport *t)
{
	lookup *new;

	if(name == NULL)
		return;

	DBG_VAL("lookup_adding",name);

	new = malloc(sizeof(lookup));
	new->name = strdup(name);
	new->t = t;
	new->next = etc.lookups;
	etc.lookups = new;
}

lookup *lookup_get(char *name)
{
	lookup *lu = etc.lookups;

	if(name == NULL)
		return NULL;

	DBG_VAL("lookup_fetching",name);

	while(lu != NULL)
	{
		if(strcmp(name, lu->name) == 0)
			return lu;
		lu = lu->next;
	}

	return NULL;
}

transport *lookup_gettransport(char *name)
{
	lookup *lu;

	lu = lookup_get(name);
	if(lu == NULL)
		return NULL;
	return lu->t;
}

transport *transport_get(conn *c)
{
	transport *next = etc.transports;

	DBG_MSG("Fetching Transport");

	while(next != NULL)
	{
		if(next->c == c)
			return next;
		next = next->next;
	}
	return NULL;
}

void transport_add(conn *c, int NEW)
{
	transport *new;

	if(c == NULL)
		return;

	DBG_VAL("transport_adding",c->name);

	new = malloc(sizeof(transport));
	new->c = c;
	new->flag_new = NEW;
	if(c->ver != NULL && strcmp(c->ver, "local") == 0)
		new->type = TRANSPORT_LOCAL;
	else
		new->type = TRANSPORT_NET;
	new->activity = malloc(sizeof(time_t));
	new->start = malloc(sizeof(time_t));
	time(new->activity);
	time(new->start);
	new->next = etc.transports;
	etc.transports = new;
	c->packets = packet_add(c->packets,"<?xml version=\"1.0\"?><ether>",NULL);
}

void transport_del(transport *t)
{
	lookup *lu, *prev;
	transport *loop, *tprev;

	if(t == NULL)
		return;

	DBG_MSG("Deleting Transport");

	lu = etc.lookups;
	prev = NULL;
	while(lu != NULL)
	{
		if(lu->t != NULL && lu->t == t)
		{
			if(prev == NULL)
				etc.lookups = lu->next;
			else
				prev->next = lu->next;

			free(lu->name);
			free(lu);

			/* we deleted one, start over and look for another */
			lu = etc.lookups;
			prev = NULL;
			continue;
		}
		prev = lu;
		lu = lu->next;
	}

	/* now lets remove the actual transport */
	loop = etc.transports;
	tprev = NULL;
	while(loop != NULL)
	{
		if(loop == t)
		{
			if(tprev == NULL)
				etc.transports = loop->next;
			else
				tprev->next = loop->next;
			free(t->activity);
			free(t->start);
		}
		loop = loop->next;
	}
}

transport *resolve(char *name)
{
	transport *t;
	struct in_addr *addr;

	if(name == NULL)
		return NULL;

	DBG_VAL("lookup_resolving",name);

	t = lookup_gettransport(name);
	if(t != NULL)
		return t;

	addr = make_addr(name);
	if(addr == NULL)
		return NULL;

	DBG_VAL("lookup_resolved_address",inet_ntoa(*addr));
	t = lookup_gettransport(inet_ntoa(*addr));
	if(t != NULL)
	{
		DBG_MSG("Found Existing Transport");
		lookup_add(name, t); /* this name resolves to this IP/transport, cache that */
		return t;
	}

	DBG_MSG("Resolution Failed!");
	return NULL;
}

transport *transport_connect(char *name)
{
	transport *t;
	struct in_addr *addr, *addr2;
	int s;
	conn *c = NULL;

	if(name == NULL)
		return NULL;

	addr = make_addr(name);
	addr2 = make_addr(DEFAULT_LOCAL);
	if(addr == NULL || addr->s_addr == addr2->s_addr) /* don't start a new connection to ourself! */
		return NULL;

	DBG_MSG("Connecting Transport");
	s = make_netsocket(DEFAULT_PORT, name, 1);
	if(s > 0)
	{
		c = conn_add(c, s);
		c->name = strdup(name);
		transport_add(c, 1);
		lookup_add(name, etc.transports);
		lookup_add(inet_ntoa(*addr), etc.transports); /* so this transport can be reused for multiple names->one ip */
		return etc.transports;
	}

	DBG_MSG("Lookup Failed!");
	return NULL;
}


