#ifndef __COMMON_FUNCTIONS_H
#define __COMMON_FUNCTIONS_H

void h_err(int check, char *err); /* crap to be removed */
void syslog(int level, char *from, char *message); /* print error log */
void syslog_debug(int level, char *from, char *name, char *val); /* convert debug messages */

jpair *jpair_add(jpair *p, char *key, char *val); /* !!! key and val must have been malloc'd!!! */
jpair *jpair_new(jpair *p, char *key, char *val, int flag); /* !!! key and val will be copied into new memory */
jpair *jpair_get(jpair *p, char *key);
char *jpair_getval(jpair *p, char *key);
char *jpair_getvalnew(jpair *p, char *key);
int jpair_getflag(jpair *p, char *key);
jpair *jpair_dup(jpair *p);
void free_jpair(jpair *p);
jpair *arg2jpair(int argc, char **argv);
jpair *str2jpair(char *str); /* turns xml attributes(TEST='asdf' foo='b a r') into jpairs */
jpair *atts2jpair(char **atts); /* turns expat **atts into jpair */
jpair *xpt2jpair(xpt *x);
void jpair_print(jpair *p); /*debug function*/
jpair *jpair_append(jpair *j1, jpair *j2);

/************************************************************/
/*
 *	Takes string "This is a test string"
 *	and start "This"
 *	and end "string"
 *	returning a pointer to a new string consisting of
 *	" is a test "
 */
char *substr(char *s, char *start, char *end);

/*
 *	Takes string
 *	"This is a test string"
 *	 ^             ^
 *	 s             end
 *	returning a pointer to a new string consisting of
 *	"This is a test"
 */
char *subpstr(char *s, char *end);

/*
 *	Counts the number of times sub appears in the string
 */
int strstrcount(char *s, char *sub);

/*
 *	Returns a new string with all occurences of old replaced with new
 *	(laymans regular expressions)
 */
char *strrepl(char *s, char *oldstr, char *newstr);

/*
 *	Grows old to contain new, can free the given old/new strings(convenient).
 *	Buffered mallocs at 100 characters, maybe it will be faster...?
 */
char *strgrow(char *oldstr, char *newstr, int old_free, int new_free);

/*
 *	escape/unescape special XML characters in a string
 */
char *strescape(const char *s);
char *strdescape(char *s);


/************************************************************/
/*
 *      Create new listen sockets for IP or unix files
 */

#ifndef WIN32
int make_netsocket(u_short port, char *host, int type);
struct in_addr *make_addr(char *host);
#endif

/*
 *	Utils to maintain conn/packet structures
 */
conn *conn_append(conn *c1, conn *c2);
conn *conn_add(conn *c, int s);
conn *conn_del(conn *c, int s);
packet *packet_add(packet *p, char *data, char *error_to);
void free_conn(conn *c);
void free_packet(packet *p);

/*
 *	Starts the io collection loop... main hearbeat
 */
void collect_data(conn *(*handle_data)());




/************************************************************/
/*
 *      Common callbacks for Expat
 */
void expat_startElement(void *userData, const char *name, const char **atts);
void expat_endElement(void *userData, const char *name);
void expat_charData(void *userData, const char *s, int len);

/*
 * expat_*_packet
 *    These are only called for the opening "root" tag in a packetized decoding
 *
 */
void expat_startElement_packet(void *userData, const char *name, const char **atts);
void expat_endElement_packet(void *userData, const char *name);

/*
 * xpt_packet_*
 *    To process an xml "stream" into packets, call xpt_packet_new to create the right pool,
 *    and as data comes into the stream, call xpt_packet_data.  After that, check the pool->packets
 *    for a linked list of packets found in the stream.
 *
 */
xptpool *xpt_packet_new();
void xpt_packet_data(xptpool *xp, char *str, int len);

/*
 *      XML file/string parsing
 */
xpt *xpt_file(char *fname);
xpt *xpt_str(char *str);

/*
 *      internal xpt(XML Parsed Tree) utils
 */
xpt *xpt_new(xpt *parent);
xpt *xpt_new_tag(xpt *parent, char *name, char **atts);
xpt *xpt_new_cdata(xpt *parent, char *data);
xptpool *xpt_newpool();

/*
 *      xpt utils
 */
char *xpt2str(xpt *x);
char *xpt2str_chain(xpt *x);
void free_xpt(xpt *x);
void free_xptpool(xptpool *xp); 
void xpt_print(xpt *x, int detail);

#endif /* __COMMON_FUNCTIONS_H */
