#include "common.h"
#include "conn.h"
#include "queue.h"

/* conn.c
 *
 * Contains helper funcitons to maintain a simple connection list.
 *
 */

conn *conn_add(conn *c, int s)
{
	conn *c_new;

	c_new = malloc(sizeof(conn));
	c_new->id = s;
	c_new->packets = NULL;
	c_new->ver = NULL;
	c_new->name = NULL;
	c_new->type = CONN_NORMAL;
	c_new->xp = NULL;
	CREATEMUTEX (c_new->lock);
	c_new->next = c;

	return c_new;
}

conn *conn_append(conn *c1, conn *c2)
{
	conn *c_count = c1;

	if(c1 == NULL)
		return c2;
	if(c2 == NULL)
		return c1;
	while(c_count->next != NULL)
		c_count = c_count->next;
	c_count->next = c2;
	return c1;
}

conn *conn_del(conn *c, int s)
{
	conn *next, *prev, *curr;
	int first_flag = 1;

	curr = prev = c;

	while(curr != NULL)
	{
		if(curr->id == s)
		{
			next = curr->next;
			close(curr->id);	/* make sure the socket is closed */
			free_conn(curr);
			if(first_flag)
				c = next;
			else
				prev->next = next;
			return c;
		}
		first_flag = 0;
		prev = curr;
		curr = curr->next;
	}
	return c;
}

void free_conn(conn *c)
{
	packet *next, *curr;

	next = c->packets;

	while(next != NULL)
	{
		curr = next;
		next = curr->next;
		free_packet(curr);
	}

	free(c->ver);
	free(c->name);
	free_xptpool(c->xp);
	FREEMUTEX (c->lock);
	free(c);
}
