#include "common.h"
#include "packet.h"

/* 

Adds a packet into a linked-list structure of packets (p). If p is nil, return the
new packet has the head of the list, otherwise add to the tail. Mutex locking is used
to lock each of the packets has it is searched for a nil friend. This is to prevent two threads
from trying to add a packet to the same list. 

*/ 

packet *packet_add(packet *p, char *data, char *error_to)
{
	packet *p_new, *p_count;

	DBG_VAL("new_packet",data);

	p_new = malloc(sizeof(packet));

	if(data == NULL)
	{
		p_new->data = malloc(1);
		p_new->data[0] = '\0';
	}else{
		p_new->data = strdup(data);
	}

	if(error_to == NULL)
		p_new->error_to = NULL;
	else
		p_new->error_to = strdup(error_to);

	p_new->sent = 0;
	p_new->next = NULL;

	CREATEMUTEX (p_new->lock);

	if(p == NULL)
	{
		return p_new;
	}else{
		p_count = p;
		while(p_count != NULL)
		{
			LOCKMUTEX (p_count->lock);
			if(p_count->next == NULL)
			{
				p_count->next = p_new;
				UNLOCKMUTEX (p_count->lock);
				break;
			}
			UNLOCKMUTEX (p_count->lock);
			p_count = p_count->next;
		}
	}

	return p;
}

void free_packet(packet *p)
{
	free(p->data);
	free(p->error_to);
	FREEMUTEX (p->lock);
	free(p);
}

