#include "jabber.h"

conn *dispatch(char *p, conn *c, int flag)
{
	conn *c_new = NULL;
	xpt *xml, *next;
	int sock;

	DBG_VAL("dispatch_data",p);

	switch(flag)
	{
		case IO_REMOVE:
			DBG_VAL("removing_connection",c->name)
			if(c->type == CONN_LOCAL)
			{
				ERROR("etherx", "Transport connection failure, exiting");
				/* should start special loop to re-connect to transport with timeout, if reconnect, reconfigure, etc */
				sig_die();
			}else{
				handle_logout(c,NULL);
				(int)c_new = -1;
			}
			break;

		case IO_INIT:
			DBG_MSG("Initialization from IO loop");

			/* connect to the local etherx */
			sock = make_netsocket(DEFAULT_ETHERXPORT, jpair_getval(etc.vars, "local"), 1);
			if(sock < 0)
			{
				ERROR("etherx", "Unable to connect to Transport, exiting");
				sig_die();
			}
			c_new = conn_add(NULL, sock);
			c_new->type = CONN_LOCAL;
			c_new->packets = packet_add(c_new->packets,"<?xml version='1.0'?><ether><handshake>",NULL);
			c_new->packets = packet_add(c_new->packets,DEFAULT_ID,NULL);
			c_new->packets = packet_add(c_new->packets,"</handshake>",NULL);
			etc.transport = c_new; /* store globally */
			break;

		case IO_NORMAL:
			DBG_MSG("Processing Normally");
			if(c->xp == NULL) /* initialize xp if it hasn't been yet */
				c->xp = xpt_packet_new();
			xpt_packet_data(c->xp,p,strlen(p));

			/* check if this is a new unseen connection, we set c->ver after we've seen it already */
			if(c->type != CONN_LOCAL && c->xp->name != NULL && strneq(c->ver,"client"))
			{
				if(strcmp(c->xp->name,"jabber") != 0)
				{
					syslog_debug(SYSLOG_ERROR, "connection", "NO_XML_HEADER_FROM", c->name);
					/* !!! should send error message back about this */
					handle_logout(c,NULL);
					return (conn *)-1;
				}else{
					/* we're ok with this client so far, flag it thus and send a header */
					if(c->ver != NULL)
						free(c->ver);
					c->ver = strdup("client");
					deliver_connection(c);
				}
			}


			next = xml = c->xp->packets;
			while(next != NULL)
			{
				xml = next;
				next = next->next;
				xml->next = NULL; /* single it out */
				if(c->type == CONN_LOCAL)
					transport_packet(c, xml);
				else
					handle_packet(c, xml);
			}
			free_xpt(c->xp->packets);
			c->xp->packets = NULL;
			if(c->xp->flag != XPT_POOLFLAG_NONE)
			{
				switch(c->xp->flag)
				{
					case XPT_POOLFLAG_ERROR:
						syslog_debug(SYSLOG_ERROR, "connection", "INVALID_XML_FROM", c->name);
						break;
					case XPT_POOLFLAG_CLOSE:
						DBG_VAL("connection_xml_close",c->name);
						break;
					default:
						DBG_VAL("unknown_xml_error",c->name);
				}
				/* !!! should send error message back about this */
				handle_logout(c,NULL);
				(int)c_new = -1;
			}
			break;
	}

	return c_new;
}

void handle_packet(conn *c, xpt *x)
{
	char *type;

	DBG_VAL("handling_normal_packet",c->name);

	type = x->name;
	if(type == NULL) /* not a valid incoming packet */
	{
		return;
	}

	if(strcmp(type,"login") == 0)
		handle_login(c, x);
	if(strcmp(type,"logout") == 0)
		handle_logout(c, x);
	if(strcmp(type,"logout") == 0)
		handle_logout(c, x);
	if(strcmp(type,"message") == 0)
		handle_message(c, x);
	if(strcmp(type,"status") == 0)
		handle_status(c, x);
	if(strcmp(type,"roster") == 0)
		handle_roster(c, x);
}

