#include "jabber.h"

void handle_login(conn *c, xpt *x)
{
	jpair *par;
	char *user, *pass, *nick, *group, *error;
	handler *h;
	session *s;
	int auth = FAIL;

	DBG_MSG("handle_login");

	par = xpt2jpair(x->children);

	user = jpair_getval(par,"user");
	pass = jpair_getval(par,"pass");
	nick = jpair_getval(par,"nick");

	/* illegal login packet */
	if(user == NULL || pass == NULL)
		return;

	group = modules_getgroup(user);
	h = modules_gethandler_group(API_AUTH,group);
	if(h != NULL)
		auth = (h->h_auth)(user, pass);
	if(auth != PASS)
	{
		DBG_MSG("Authentication Failed");
		error = strgrow(NULL,"<message type=\"error\" for=\"",0,0);
		error = strgrow(error,user,1,0);
		error = strgrow(error,"\" nick=\"",1,0);
		error = strgrow(error,nick,1,0);
		error = strgrow(error,"\"><subject>Login Failed</subject><say>The server could not validate your username and password.  Please make sure you typed it in correctly and try again, or contact the server administrator.</say><ext>LoginFailed</ext></message>",1,0);
		c->packets = packet_add(c->packets,error,NULL);
		free(error);
	}else{
		s = session_add(c,user,nick,group);
		h = modules_gethandler_group(API_LOGIN,group);
		if(h != NULL && h->h_log != NULL)
			(h->h_log)(user, nick);
	}
	free_jpair(par);
	free(group);
}

void handle_logout(conn *c, xpt *x)
{
	session *s, *next;
	char *user;

	DBG_MSG("handle_logout");

	if(x == NULL || c->type == CONN_NORMAL)
	{
		/* manual call, must have been error on connection, log all sessions off and close it */
		s = session_lookup(c,NULL,NULL);
		next = s;
		while(next != NULL)
		{
			s = next;
			next = next->lookup_next;
			session_del(s);
		}
	}else{
		user = jpair_getval(x->attributes, "from");
		if(user != NULL && c->type == CONN_MULTI)
			session_del(session_lookup(c, user, NULL));
	}
}

void deliver_connection(conn *c)
{
	char *str = NULL;

	str = strgrow(str, "<?xml version='1.0'?><jabber version=\"", 1, 0);
	str = strgrow(str, APPSIG, 1, 0);
	str = strgrow(str, "\" protocol=\"", 1, 0);
	str = strgrow(str, PROTOCOL, 1, 0);
	str = strgrow(str, "\">", 1, 0);

	c->packets = packet_add(c->packets, str, NULL);

	free(str);
}
