#include "jabber.h"

void handle_message(conn *c, xpt *x)
{
	jpair *from, *to, *jnext;
	char *sub, *say, *ext, *thread, *pristr, *type;
	int priority;
	xpt *next, *next2;

	DBG_MSG("handle_message");

	if(c == NULL || x == NULL)
		return;

	from = get_from(c, x);
	to = get_to(c, x);
	if(from == NULL || to == NULL)
	{
		DBG_MSG("Invalid message, improper from or to");
		return;
	}

	pristr = sub = say = ext = thread = NULL;
	priority = 0;

	type = jpair_getvalnew(x->attributes, "type");

	next = x->children;
	while(next != NULL)
	{
		if(next->type == XPT_TAG && strcmp(next->name, "subject") == 0)
			sub = xpt2str_chain(next->children);
		if(next->type == XPT_TAG && strcmp(next->name, "say") == 0)
			say = xpt2str_chain(next->children);
		if(next->type == XPT_TAG && strcmp(next->name, "thread") == 0)
			thread = xpt2str_chain(next->children);
		if(next->type == XPT_TAG && strcmp(next->name, "ext") == 0)
		{
			next2 = next->children;
			ext = NULL;
			while(next2 != NULL)
			{
				ext = strgrow(ext, xpt2str(next2), 1, 1);
				next2 = next2->next;
			}
		}
		if(next->type == XPT_TAG && strcmp(next->name, "priority") == 0)
		{
			pristr = xpt2str_chain(next->children);
			sscanf(pristr,"%d",&priority);
		}
		next = next->next;
	}

	jnext = to;
	while(jnext != NULL)
	{
		deliver_message(from->key, from->val, jnext->key, jnext->val, sub, say, thread, ext, priority, type); 
		jnext = jnext->next;
	}

	free_jpair(from);
	free_jpair(to);
	free(sub);
	free(say);
	free(ext);
	free(thread);
	free(pristr);
	free(type);
}
 

void deliver_message(	char *from_user,
			char *from_nick,
			char *to_user,
			char *to_nick,
			char *subject,
			char *say,
			char *thread,
			char *ext,
			int priority,
			char *type)
{
	xpt *parent, *x;
	char intbuff[20], *server, *str;
	session *s, *next;
	conn *c = NULL;
	jpair *par;
	int local, stored, apistat;
	handler *h;

	if(to_user == NULL) /*WTF?*/
		return;

	local = stored = FALSE;

	parent = xpt_new_tag(NULL, "message", NULL);

	if(type != NULL)
		parent->attributes = jpair_new(parent->attributes, "type", type, 0);

	if(subject != NULL)
	{
		x = xpt_new_tag(parent, "subject", NULL);
		xpt_new_cdata(x, subject);
	}

	if(say != NULL)
	{
		x = xpt_new_tag(parent, "say", NULL);
		xpt_new_cdata(x, say);
	}

	if(thread != NULL)
	{
		x = xpt_new_tag(parent, "thread", NULL);
		xpt_new_cdata(x, thread);
	}

	if(ext != NULL)
	{
		x = xpt_new_tag(parent, "ext", NULL);
		xpt_new_cdata(x, ext);
		x->children->name = strdup("NOESCAPE"); /* kludge so that it isn't escaped automatically */
	}

	if(priority != 0)
	{
		x = xpt_new_tag(parent, "priority", NULL);
		sprintf(intbuff,"%d",priority);
		xpt_new_cdata(x, intbuff);
	}


	/* this is an outgoing message from a local user, if there is a module handler and it FAILs the message, die */
	if(strstr(from_user, "@") == NULL)
	{
		h = modules_gethandler_user(API_MESSAGEOUT, from_user);
		if(h != NULL && h->h_message != NULL)
			if((h->h_message)(from_user, from_nick, to_user, to_nick, subject, say, thread, ext, priority, type) == FAIL)
				return;
	}

	server = strstr(to_user, "@");
	if(server != NULL)
	{
		++server;

		if(islocal(server) == FALSE)
		{
			DBG_VAL("message_to_server",server);
			transport_deliver(from_user, from_nick, to_user, to_nick, parent);
			return;
		}else{
			/* one of the names for this transport matched the server address, deliver here */
			/* fix the userid */
			--server;
			*server = '\0';
		}
	}


	/* only local clients see <from></from> tag */
	x = xpt_new_tag(parent, "from", NULL);
	if(from_nick != NULL)
		x->attributes = jpair_new(x->attributes, "nick", from_nick, 0);
	xpt_new_cdata(x, from_user);

	h = modules_gethandler_user(API_MESSAGEIN, to_user);
	if(h != NULL && h->h_message != NULL)
	{
		apistat = (h->h_message)(to_user, to_nick, from_user, from_nick, subject, say, thread, ext, priority, type);
		if(apistat == IGNORE) /* ignore this message */
			return;
		if(apistat == BOUNCE) /* bounce it */
			stored = FALSE;
		if(apistat == SEND) /* module handled it */
			stored = TRUE;
	}

	next = s = session_lookup(NULL, to_user, to_nick);
	/* find the highest priority connected session for this user */
	while(next != NULL)
	{
		if(next->status_priority > s->status_priority)
			s = next;
		next = next->lookup_next;
	}
	if(s == NULL)
	{
		DBG_MSG("Message delivery failed, recipient not online");
		if(stored == FALSE)
		{
			if(type != NULL && strcmp(type,"error") == 0) /* bail out, don't create a loop */
				return;
			str = strgrow(NULL,"Your message",1,0);
			if(to_user != NULL)
			{
				str = strgrow(str," to ",1,0);
				str = strgrow(str,to_user,1,0);
			}
			str = strgrow(str," could not be delivered, either the recipient doesn't exist, or it couldn't be stored until they return online.\n\nMessage:\n",1,0);
			if(say != NULL)
				str = strgrow(str,say,1,0);
			deliver_message("system", NULL, from_user, from_nick, "Message Delivery Failed", str, thread, "DeliveryFailed", 0, "error");
			free(str);
		}
		return;
	}
	local_deliver(from_user, to_user, to_nick, s->c, parent);

	free_xpt(parent);
}
