#include "jabber.h"

handle_roster(conn *c, xpt *x)
{
	jpair *from, *send;
	xpt *next;
	handler *h;
	int type;
	char *group, *user;

	DBG_MSG("handle_roster");

	if(c == NULL || x == NULL || c->type == CONN_LOCAL)
		return;

	from = get_from(c, x);
	if(from == NULL)
	{
		DBG_MSG("Invalid roster, unknown or illegal sender");
		return;
	}

	h = modules_gethandler_user(API_ROSTER, from->key);
	if(h == NULL || h->h_roster == NULL)
	{
		DBG_MSG("Invalid roster, no handler defined");
		return;
	}

	send = NULL;
	next = x->children;
	while(next != NULL)
	{
		type = ROSTER_NONE;
		if(next->type == XPT_TAG && strcmp(next->name, "add") == 0)
			type = ROSTER_ADD;
		if(next->type == XPT_TAG && strcmp(next->name, "del") == 0)
			type = ROSTER_DEL;
		if(next->type == XPT_TAG && strcmp(next->name, "get") == 0)
			type = ROSTER_GET;
		if(type != ROSTER_NONE)
		{
			group = jpair_getval(next->attributes, "group");
			user = xpt2str_chain(next->children);
			send = jpair_append(send, (h->h_roster)(from->key, type, group, user));
			free(user);
		}
		next = next->next;
	}

	if(send != NULL)
		deliver_roster(c, from->key, from->val, send);
	free_jpair(send);
	free_jpair(from);
}


void deliver_roster(conn *c, char *user, char *nick, jpair *list)
{
	xpt *x, *new;

	DBG_MSG("deliver_roster");

	x = xpt_new_tag(NULL, "roster", NULL);
	while(list != NULL)
	{
		new = xpt_new_tag(x, "group", NULL);
		new->attributes = jpair_new(NULL, "name", list->key, 0);
		xpt_new_cdata(new, list->val);
		list = list->next;
	}

	local_deliver(NULL, user, nick, c, x);
}
