#include "jabber.h"

handle_status(conn *c, xpt *x)
{
	jpair *from, *to, *jnext;
	char *say, *ext, *icon, *pristr, *statstr;
	int status, priority;
	xpt *next, *next2;

	DBG_MSG("handle_status");

	if(c == NULL || x == NULL)
		return;

	from = get_from(c, x);
	to = get_to(c, x);
	if(from == NULL)
	{
		DBG_MSG("Invalid status, improper from");
		return;
	}

	status = STATUS_NORMAL;
	statstr = jpair_getval(x->attributes, "type");
	if(statstr != NULL && strcmp(statstr, "online") == 0)
		status = STATUS_ONLINE;
	if(statstr != NULL && strcmp(statstr, "offline") == 0)
		status = STATUS_OFFLINE;

	pristr = icon = say = ext = NULL;
	priority = 0;

	next = x->children;
	while(next != NULL)
	{
		if(next->type == XPT_TAG && strcmp(next->name, "say") == 0)
			say = xpt2str_chain(next->children);
		if(next->type == XPT_TAG && strcmp(next->name, "icon") == 0)
			icon = xpt2str_chain(next->children);
		if(next->type == XPT_TAG && strcmp(next->name, "ext") == 0)
		{
			next2 = next->children;
			ext = NULL;
			while(next2 != NULL)
			{
				ext = strgrow(ext, xpt2str(next2), 1, 1);
				next2 = next2->next;
			}
		}
		if(next->type == XPT_TAG && strcmp(next->name, "priority") == 0)
		{
			pristr = xpt2str_chain(next->children);
			sscanf(pristr,"%d",&priority);
		}
		next = next->next;
	}

	if(c->type == CONN_LOCAL)
	{
		/* this is an incoming remote status update */
		jnext = to;
		while(jnext != NULL)
		{
			status_update(from->key, from->val, jnext->key, status, say, icon, ext, priority); 
			jnext = jnext->next;
		}
	}else{
		status_change(from->key, from->val, status, say, icon, ext, priority);
	}

	free_jpair(from);
	free_jpair(to);
	free(icon);
	free(say);
	free(ext);
	free(pristr);
}


void status_update(char *user, char *nick, char *to, int status, char *say, char *icon, char *ext, int priority)
{ 
	xpt *parent, *x;
	char intbuff[20], *server;
	session *s;
	handler *h;
	int sendstatus = FAIL;

	parent = xpt_new_tag(NULL, "status", NULL);

	switch(status)
	{
		case STATUS_NORMAL:
			break;
		case STATUS_ONLINE:
			parent->attributes = jpair_new(NULL, "type", "online", 0);
			break;
		case STATUS_OFFLINE:
			parent->attributes = jpair_new(NULL, "type", "offline", 0);
			break;
	}

	if(say != NULL)
	{
		x = xpt_new_tag(parent, "say", NULL);
		xpt_new_cdata(x, say);
	}

	if(icon != NULL)
	{
		x = xpt_new_tag(parent, "icon", NULL);
		xpt_new_cdata(x, icon);
	}

	if(ext != NULL)
	{
		x = xpt_new_tag(parent, "ext", NULL);
		xpt_new_cdata(x, ext);
		x->children->name = strdup("NOESCAPE"); /* kludge so that it isn't escaped automatically */
	}

	if(priority != 0)
	{
		x = xpt_new_tag(parent, "priority", NULL);
		sprintf(intbuff,"%d",priority);
		xpt_new_cdata(x, intbuff);
	}


	server = strstr(to, "@");
	if(server != NULL)
	{
		++server;

		if(islocal(server) == FALSE)
		{
			DBG_VAL("status_to_server",server);
			transport_deliver(user, nick, to, NULL, parent);
			return;
		}else{
			/* fix the userid */
			--server;
			*server = '\0';
		}
	}


	/* only local client see the <from></from> tag */
	x = xpt_new_tag(parent, "from", NULL);
	if(nick != NULL)
		x->attributes = jpair_new(x->attributes, "nick", nick, 0);
	xpt_new_cdata(x, user);

	h = modules_gethandler_user(API_STATUSIN, to);
	if(h != NULL && h->h_status_in != NULL)
	{
		sendstatus = (h->h_status_in)(to, user, nick, status, say, icon, ext, priority);
		if(sendstatus == FAIL)
			return;
	}

	/* find the local user this is to, and send the status to all of their sessions */
	s = session_lookup(NULL, to, NULL);
	while(s != NULL)
	{
		if(s->status_flag != STATUS_OFFLINE && !(streq(user, s->user) && streq(nick, s->nick)))
		{
			local_deliver(user, s->user, s->nick, s->c, parent);
			/* if we're allowed to, reply with the recipients current status */
			if(sendstatus != PRIVATE && status == STATUS_ONLINE)
				status_update(s->user, s->nick, user, s->status_flag, s->status_say, s->status_icon, s->status_ext, s->status_priority);
		}
		s = s->next;
	}

	free_xpt(parent);
}


void status_change(char *user, char *nick, int status, char *say, char *icon, char *ext, int priority)
{
	handler *h;
	jpair *notify = NULL;
	session *s;

	s = session_lookup(NULL, user, nick);
	if(s == NULL)
		return;

	/* store status data for this session */
	free(s->status_say);
	free(s->status_icon);
	free(s->status_ext);

	s->status_say = (say == NULL)?NULL:strdup(say);
	s->status_icon = (icon == NULL)?NULL:strdup(icon);
	s->status_ext = (ext == NULL)?NULL:strdup(ext);
	s->status_priority = priority;
	if(status == STATUS_ONLINE)	/* online status is only a special flag, real status is "normal" or offline */
		s->status_flag = STATUS_NORMAL;
	else
		s->status_flag = status;

	h = modules_gethandler_user(API_STATUSOUT, user);
	if(h != NULL && h->h_status_out != NULL)
	{
		notify = (h->h_status_out)(user, nick, status, say, icon, ext, priority);
	}

	while(notify != NULL)
	{
		status_update(user, nick, notify->key, status, say, icon, ext, priority);
		notify = notify->next;
	}

	/* always tell this users other sessions about the new status */
	status_update(user, nick, user, status, say, icon, ext, priority);
}


