#include "jabber.h"

struct basic_list
{
	char *id;
	int status;
	struct basic_list *next;
};

struct basic_roster
{
	char *id;
	char *group;
	struct basic_list *members;
	struct basic_roster *next;
};

struct basic_roster *basic_rosters = NULL;
jpair *basic_authusers = NULL;
jpair *basic_config = NULL;

void basic_fromfile();
void basic_tofile();

int basic_getlist(char *id, struct basic_list *list)
{
	DBUG("MOD_BASIC: getlist for id ",id)

	if(id == NULL || list == NULL)
		return 0;
	while(list != NULL)
	{
		if(strcmp(id, list->id) == 0)
			return 1;
		list = list->next;
	}
	return 0;
}

struct basic_list *basic_setlist(char *id, struct basic_list *list)
{
	struct basic_list *new;

	DBUG("MOD_BASIC: setlist for id ",id)

	if(id == NULL)
		return NULL;
	if(basic_getlist(id,list) == 0)
	{
		new = malloc(sizeof(struct basic_list));
		new->id = strdup(id);
		new->status = 0;
		new->next = list;
		return new;
	}
	return list;
}

struct basic_list *basic_freelist(char *id, struct basic_list *list)
{
	struct basic_list *prev, *next;
	int special = 0;

	DBUG("MOD_BASIC: freelist for id ",id)

	prev = next = list;

	if(list != NULL && list->next == NULL)
		special = 1;

	if(id == NULL)
		return NULL;
	while(next != NULL)
	{
		if(strcmp(id, next->id) == 0)
		{
			prev->next = next->next;
			free(next->id);
			free(next);
			if(special)
				return NULL;
			return list;
		}
		prev = next;
		next = next->next;
	}
	return list;
}

void basic_rosteradd(char *id, char *group, char *newid)
{
	struct basic_roster *new, *all = basic_rosters;

	DBUG("MOD_BASIC: rosteradd for id ",id)

	if(id == NULL || group == NULL || newid == NULL)
		return;

	basic_tofile();

	while(all != NULL)
	{
		if(strcmp(id, all->id) == 0 && strcmp(group, all->group) == 0)
		{
			all->members = basic_setlist(newid,all->members);
			return;
		}
		all = all->next;
	}
	new = malloc(sizeof(struct basic_roster));
	new->id = strdup(id);
	new->group = strdup(group);
	new->members = basic_setlist(newid,NULL);
	new->next = basic_rosters;
	basic_rosters = new;
}

void basic_rosterdel(char *id, char *group, char *newid)
{
	struct basic_roster *all = basic_rosters;

	DBUG("MOD_BASIC: rosterdel for id ",id)

	if(id == NULL || group == NULL || newid == NULL)
		return;

	basic_tofile();

	while(all != NULL)
	{
		if(strcmp(id, all->id) == 0 && strcmp(group, all->group) == 0)
		{
			all->members = basic_freelist(newid,all->members);
			return;
		}
		all = all->next;
	}
}

jpair *basic_jpairgroup(char *id, char *group)
{
	struct basic_roster *all = basic_rosters;
	struct basic_list *next;
	jpair *new = NULL, *par = NULL;

	DBUG("MOD_BASIC: create jpair from group for id ",id)

	if(id == NULL)
		return NULL;

	if(group == NULL)
	{
		while(all != NULL)
		{
			if(strcmp(id, all->id) == 0)
			{
				new = malloc(sizeof(jpair));
				new->key = strdup(all->group);
				new->val = NULL;
				new->next = par;
				par = new;
			}
			all = all->next;
		}
		return par;
	}

	while(all != NULL)
	{
		if(strcmp(id, all->id) == 0 && strcmp(group, all->group) == 0)
		{
			next = all->members;
			while(next != NULL)
			{
				new = malloc(sizeof(jpair));
				new->key = strdup(group);
				new->val = strdup(next->id);
				new->next = par;
				par = new;
				next = next->next;
			}
			return par;
		}
		all = all->next;
	}
	return NULL;
}




void basic_tofile()
{
	int fd, check;
	char *xml, *file;
	struct basic_roster *nextr;
	struct basic_list *nextl;

	/* just for rudimentary storage of basic rosters at the moment */

	if(jpair_getval(basic_config,"spool") == NULL)
	{
		syslog(SYSLOG_DEBUG,"mod_basic","No spool defined, not storing anything permanently");
		return;
	}
	file = strgrow(NULL,jpair_getval(basic_config,"spool"),0,0);
	file = strgrow(file,"/rosters.xml",1,0);
	unlink(file);
	fd = open(file, O_WRONLY | O_CREAT, S_IRWXU);
	if(fd < 0)
	{
		syslog_debug(SYSLOG_ERROR,"mod_basic","spool_file_failure",file);
		return;
	}

	xml = strgrow(NULL,"<rosters>",0,0);
	nextr = basic_rosters;
	while(nextr != NULL)
	{
		xml = strgrow(xml,"<group id=\"",1,0);
		xml = strgrow(xml,strescape(nextr->group),1,1);
		xml = strgrow(xml,"\" user=\"",1,0);
		xml = strgrow(xml,strescape(nextr->id),1,1);
		xml = strgrow(xml,"\">",1,0);

		nextl = nextr->members;
		while(nextl != NULL)
		{
			xml = strgrow(xml,"<user>",1,0);
			xml = strgrow(xml,strescape(nextl->id),1,1);
			xml = strgrow(xml,"</user>",1,0);
			nextl = nextl->next;
		}

		xml = strgrow(xml,"</group>",1,0);
		nextr = nextr->next;
	}
	xml = strgrow(xml,"</rosters>",1,0);

	check = write(fd, xml, strlen(xml));
	if(check < 0)
		syslog(SYSLOG_ERROR,"mod_basic","Writing to roster spool file failed");
	close(fd);
	free(xml);
}

void basic_fromfile()
{
	int fd;
	xpt *x, *next, *next2;
	char *user, *file;

	if(jpair_getval(basic_config,"spool") == NULL)
	{
		syslog(SYSLOG_DEBUG,"mod_basic","No spool defined, not storing anything permanently");
		return;
	}
	file = strgrow(NULL,jpair_getval(basic_config,"spool"),0,0);
	file = strgrow(file,"/rosters.xml",1,0);

	fd = open(file,O_RDONLY);
	if(fd < 0)
	{
		syslog_debug(SYSLOG_ERROR,"mod_basic","spool_file_failure",file);
	}else{
		x = xpt_file(file);

		/* icky but works, decoding the xpt */
		if(x != NULL)
		{
			/* <rosters> */
			next = x->children;
			while(next != NULL)
			{
				if(next->type == XPT_TAG)
				{
					/* <group id="" user=""> */
					next2 = next->children;
					while(next2 != NULL)
					{
						if(next2->type == XPT_TAG)
						{
							/* <user>sdfasd</user> */
							user = xpt2str_chain(next2->children);
							basic_rosteradd(jpair_getval(next->attributes,"user"), jpair_getval(next->attributes,"id"), user);
							free(user);
						}
						next2 = next2->next;
					}
				}
				next = next->next;
			}
		}else{
			syslog_debug(SYSLOG_ERROR,"mod_basic","spool_file_parsing_failure",file);
		}
	}

	close(fd);
}



void basic_login(char *user, char *nick)
{
	jpair *par;
	int fd;
	xpt *x, *next;
	char *file;

	DBUG("MOD_BASIC: login!",user);

	par = jpair_get(basic_authusers, user);
	if(par == NULL)
		return; /* not our user! */

	/* session count for this user */
	par->flag++;

	if(par->flag != 1)
		return;

	/* fetch any offline messages */

	if(jpair_getval(basic_config,"spool") == NULL)
		return;

	file = strgrow(NULL,jpair_getval(basic_config,"spool"),0,0);
	file = strgrow(file,"/",1,0);
	file = strgrow(file,user,1,0);
	file = strgrow(file,".offline.xml",1,0);
	fd = open(file, O_RDONLY);
	if(fd < 0)
	{
		syslog_debug(SYSLOG_ERROR,"mod_basic","spool_file_failure",file);
		return;
	}

	x = xpt_file(file);
	if(x == NULL)
		return;

	next = x->children;
	while(next != NULL)
	{
		par = xpt2jpair(next->children);
		deliver_message(jpair_getval(par,"fromid"),
				jpair_getval(par,"fromnick"),
				user,nick,
				jpair_getval(par,"sub"),
				jpair_getval(par,"say"),
				jpair_getval(par,"thread"),
				jpair_getval(par,"ext"),0,
				jpair_getval(par,"type"));
		next = next->next;
	}

	close(fd);
	unlink(file);
	free_xpt(x);
}

void basic_logout(char *user, char *nick)
{
	jpair *par;

	DBUG("MOD_BASIC: logout!",user);

	par = jpair_get(basic_authusers, user);
	if(par == NULL)
		return; /* not our user! */

	/* session count for this user */
	par->flag--;
}

int mod_basic_auth(char *id, char *pass)
{
	DBUG("MOD_BASIC: handler_auth ",id)
	if(jpair_getval(basic_authusers,id) != NULL)
	{
		if(strcmp(pass, jpair_getval(basic_authusers,id)) == 0)
		{
			DBUG("MOD_BASIC: AUTH SUCCESSFUL!","")
			return PASS;
		}else{
			DBUG("MOD_BASIC: AUTH FAILED!","")
			return FAIL;
		}
	}
	DBUG("MOD_BASIC: AUTH NOT FOUND","")
	return FAIL;
}


int mod_basic_statusin(char *to, char *user, char *nick, int type, char *say, char *icon, char *ext, int priority)
{
	struct basic_roster *next;
	struct basic_list *nextl;

	DBG_VAL("MOD_BASIC_statusin_to!",to);

	if(user == NULL || to == NULL) /* incomplete, just deliver by default */
		return SEND;

	next = basic_rosters;
	while(next != NULL)
	{
		if(next->id != NULL && strcmp(next->id, to) == 0)
		{
			nextl = next->members;
			while(nextl != NULL)
			{
				if(nextl->id != NULL && strcmp(nextl->id, user) == 0)
				{
					nextl->status = 1;
					DBG_VAL("MOD_BASIC_statusin_from_saved!",user);
				}
				nextl = nextl->next;
			}
		}
		next = next->next;
	}

	return SEND;
}

jpair *mod_basic_statusout(char *user, char *nick, int type, char *say, char *icon, char *ext, int priority)
{
	struct basic_roster *next;
	struct basic_list *nextl;
	jpair *ret;

	DBG_VAL("MOD_BASIC_statusout_from!",user);

	if(user == NULL)
		return NULL;

	ret = NULL;

	next = basic_rosters;
	while(next != NULL)
	{
		if(next->id != NULL && strcmp(next->id, user) == 0)
		{
			nextl = next->members;
			while(nextl != NULL)
			{
				if(nextl->status == 1 || type == STATUS_ONLINE)
				{
					DBG_VAL("MOD_BASIC_statusout_to!",nextl->id);
					ret = jpair_new(ret, nextl->id, NULL, 0);
				}
				nextl = nextl->next;
			}
		}
		next = next->next;
	}

	return ret;
}

jpair *mod_basic_roster(char *localuser, int type, char *group, char *user)
{
	DBUG("MOD_BASIC: handler_roster ",localuser)
	switch(type)
	{
		case ROSTER_DEL:
			basic_rosterdel(localuser, group, user);
			break;
		case ROSTER_ADD:
			basic_rosteradd(localuser, group, user);
			break;
		case ROSTER_GET:
			return basic_jpairgroup(localuser,group);
			break;
	}
	return NULL;
}


int mod_basic_messagein(char *localuser, char *localnick, char *otheruser, char *othernick, char *subject, char *say, char *thread, char *ext, int priority, char *type)
{
	jpair *par;
	int fd, check;
	char *xml, *file;
	xpt *x, *msg, *tag;

	DBG_VAL("MOD_BASIC_messagein",localuser);

	par = jpair_get(basic_authusers, localuser);
	if(par == NULL)
		return BOUNCE; /* not our user! */
	if(par->flag > 0)
		return SEND;

	/* just for rudimentary storage of basic offline messages at the moment */

	if(jpair_getval(basic_config,"spool") == NULL)
	{
		syslog(SYSLOG_DEBUG,"mod_basic","No spool defined, not storing anything permanently");
		return BOUNCE;
	}

	file = strgrow(NULL,jpair_getval(basic_config,"spool"),0,0);
	file = strgrow(file,"/",1,0);
	file = strgrow(file,localuser,1,0);
	file = strgrow(file,".offline.xml",1,0);
	fd = open(file, O_RDWR | O_CREAT, S_IRWXU);
	if(fd < 0)
	{
		syslog_debug(SYSLOG_ERROR,"mod_basic","spool_file_failure",file);
		return BOUNCE;
	}

	x = xpt_file(file);
	if(x == NULL)
		x = xpt_new_tag(NULL,"messages",NULL);
	msg = xpt_new_tag(x, "message", NULL);

	tag = xpt_new_tag(msg, "fromid", NULL);
	xpt_new_cdata(tag,otheruser);
	tag = xpt_new_tag(msg, "fromnick", NULL);
	xpt_new_cdata(tag,othernick);
	tag = xpt_new_tag(msg, "subject", NULL);
	xpt_new_cdata(tag,subject);
	tag = xpt_new_tag(msg, "say", NULL);
	xpt_new_cdata(tag,say);
	tag = xpt_new_tag(msg, "thread", NULL);
	xpt_new_cdata(tag,thread);
	tag = xpt_new_tag(msg, "ext", NULL);
	xpt_new_cdata(tag,ext);
	tag = xpt_new_tag(msg, "type", NULL);
	xpt_new_cdata(tag,type);

	xml = xpt2str(x);
	check = write(fd, xml, strlen(xml));
	if(check < 0)
		syslog(SYSLOG_ERROR,"mod_basic","Writing to roster spool file failed");

	close(fd);
	free_xpt(x);
	free(xml);

	return SEND;
}

int mod_basic_messageout(char *localuser, char *localnick, char *otheruser, char *othernick, char *subject, char *say, char *thread, char *ext, int priority, char *type)
{
	/* Just to notify the module that the user sent an outgoing message to another user
	 * -- return SEND/FAIL to tell the server to actually deliver the message
	 */
	DBG_VAL("MOD_BASIC_messageout",localuser);
	return SEND;
}

void mod_basic_init(xpt *config)
{
	char *pass;
	xpt *next;
	jpair *j;

	DBUG("MOD_BASIC: init!","");

	/* finding <user>username@password</user> special fields and storing them, mostly for testing purposes */
	next = config;
	while(next != NULL)
	{
		if(next->type == XPT_TAG && strcmp(next->name,"mod_basic") == 0)
		{
			j = xpt2jpair(next->children);
			basic_config = j; /* store for later reference */
			while(j != NULL)
			{
				if(strcmp(j->key,"user") == 0)
				{
					DBUG("MOD_BASIC: initializing user database from config","")
					pass = strstr(j->val,"@");
					*pass = '\0';
					++pass;
					basic_authusers = jpair_new(basic_authusers,j->val,pass,0);
				}

				j = j->next;
			}
		}
		next = next->next;
	}

	/* initialize critical vars if not set */
	if(jpair_get(basic_config,"spool") == NULL)
		basic_config = jpair_new(basic_config,"spool",DEFAULT_VAR,0);

	module_set_auth("mod_basic",mod_basic_auth);
	module_set_log("mod_basic",basic_login,basic_logout);
	module_set_roster("mod_basic",mod_basic_roster);
	module_set_status("mod_basic", mod_basic_statusin, mod_basic_statusout);
	module_set_message("mod_basic", mod_basic_messagein, mod_basic_messageout);

	basic_fromfile();

	DBUG("MOD_BASIC: returning","");
}



