#include "jabber.h"

char *mod_test_group(char *user)
{
	/* Returns the "group" that the user belongs to, only if we "know" about this user */
	DBG_VAL("MOD_TEST_getgroup",user)
	/* return strdup("test"); */
	return NULL;
}

int mod_test_auth(char *user, char *pass)
{
	/* Authentication handler, returns PASS only if the username/password check out */
	DBG_VAL("MOD_TEST_auth",user)
	return PASS;
}

void mod_test_login(char *user, char *nick)
{
	/* Just to notify the module that the user logged in, general purpose
	 */
	DBG_VAL("MOD_TEST_login",user);
}

void mod_test_logout(char *user, char *nick)
{
	/* Just to notify the module that the user logged out, general purpose
	 */
	DBG_VAL("MOD_TEST_logout",user);
}

int mod_test_messagein(char *localuser, char *localnick, char *otheruser, char *othernick, char *subject, char *say, char *thread, char *ext, int priority, char *type)
{
	/* Just to notify the module that the user is recieving a message
	 * SPECIAL: if this user isn't online(based on login/logout handlers) store it for them, offline message
	 * -- return SEND/BOUNCE/IGNORE to tell the server to actually deliver the message
	 * SEND the server will send it, IGNORE the server silently ignores it, BOUNCE the server will bounce it
	 */
	DBG_VAL("MOD_TEST_messagein",localuser);
	return SEND;
}

int mod_test_messageout(char *localuser, char *localnick, char *otheruser, char *othernick, char *subject, char *say, char *thread, char *ext, int priority, char *type)
{
	/* Just to notify the module that the user sent an outgoing message to another user
	 * -- return SEND/FAIL/UNKNOWN to tell the server to actually deliver the message
	 * SEND the server will send it, IGNORE the server silently ignores it, BOUNCE the server will bounce it
	 */
	DBG_VAL("MOD_TEST_messageout",localuser);
	return SEND;
}

int mod_test_statusin(char *to, char *user, char *nick, int type, char *say, char *icon, char *ext, int priority)
{
	/* Called whenever the user is receiving a status change from another user.
	 * to = the local user that will be receiving the status change
	 * user/nick = the ID of the user sending the status change
	 * -- return SEND to tell the server to actually deliver the status change
	 * 	(if it's a type=STATUS_ONLINE the server will also send back this users status)
	 * -- return FAIL to completly ignore this status change
	 * -- return PRIVATE to ONLY deliver it to the user(don't respond to the sender for online status)
	 */
	DBG_VAL("MOD_TEST_statusin_from!",user);
	DBG_VAL("MOD_TEST_statusin_to!",to);
	return SEND;
}

jpair *mod_test_statusout(char *user, char *nick, int type, char *say, char *icon, char *ext, int priority)
{
	/* Called whenever the user is changing their status and sending the change.
	 * user/nick = the ID of the user sending the status change
	 * -- return a jpair list containing the ID's of users to deliver the status change to
	 */
	DBG_VAL("MOD_TEST_statusout_from!",user);
	return NULL;
}

jpair *mod_test_roster(char *localuser, int type, char *group, char *user)
{
	/* Called to add/del and fetch roster entries for this user
	 * -- return a jpair list containing the group name and user id for each roster entry, or NULL
	 */
	DBG_VAL("MOD_TEST_roster_localuser",localuser);
	DBG_VAL("MOD_TEST_roster_group",group);
	DBG_VAL("MOD_TEST_roster_user",user);
	if(type == ROSTER_GET)
	{
		return jpair_new(NULL, "testgroup", "myfriend", 0);
	}
	return NULL;
}


void mod_test_init(xpt *config)
{
	char *pass;
	xpt *next;
	jpair *j;

	DBG_MSG("MOD_TEST_init");

	module_set_group("mod_test", mod_test_group);
	module_set_auth("mod_test", mod_test_auth);
	module_set_log("mod_test", mod_test_login, mod_test_logout);
	module_set_status("mod_test", mod_test_statusin, mod_test_statusout);
	module_set_message("mod_test", NULL, mod_test_messageout);
	module_set_roster("mod_test", mod_test_roster);
}


