#include "jabber.h"

session *session_add(conn *c, char *user, char *nick, char *group)
{
	session *s;

	DBG_VAL("new_session",user)

	if(session_lookup(c,NULL,NULL) != NULL)
		c->type = CONN_MULTI;

	s = malloc(sizeof(session));
	s->c = c;

	if(user == NULL)
		s->user = NULL;
	else
		s->user = strdup(user);

	if(nick == NULL)
		s->nick = NULL;
	else
		s->nick = strdup(nick);

	if(group == NULL)
		s->group = NULL;
	else
		s->group = strdup(group);

	s->status_say = NULL;
	s->status_icon = NULL;
	s->status_ext = NULL;
	s->status_flag = STATUS_OFFLINE;
	s->status_priority = 0;
	s->activity = malloc(sizeof(time_t));
	time(s->activity);
	s->start = malloc(sizeof(time_t));
	time(s->start);
	s->lookup_next = NULL;

	s->next = etc.sessions;
	etc.sessions = s;
	return s;
}

void session_del(session *s)
{
	session *prev, *next, *curr;
	int first_flag = 1;
	handler *h;

	DBG_VAL("removing_session ",s->user)
	curr = prev = etc.sessions;

	while(curr != NULL)
	{
		if(curr == s)
		{
			next = curr->next;

			/* send offline status update for this user */
			status_change(s->user, s->nick, STATUS_OFFLINE, "Offline", "offline", NULL, 0);

			h = modules_gethandler_group(API_LOGOUT,s->group);
			if(h != NULL && h->h_log != NULL)
				(h->h_log)(s->user, s->nick);

			free_session(s);
			if(first_flag)
				etc.sessions = next;
			else
				prev->next = next;
			return;
		}
		first_flag = 0;
		prev = curr;
		curr = curr->next;
	}
}

session *session_lookup(conn *c, char *user, char *nick) /* returns multiple sessions */
{
	session *curr, *ret, *ret_end;

	DBG_VAL("lookup_session",user)

	curr = etc.sessions;
	ret = ret_end = NULL;

	while(curr != NULL)
	{
		curr->lookup_next = NULL;
		if(c == NULL)
		{
			if(user != NULL && curr->user != NULL && strcmp(user,curr->user) == 0)
			{
				if(nick != NULL && curr->nick != NULL && strcmp(nick,curr->nick) == 0)
					return curr;
				if(ret == NULL) /* if this is the first, start there */
				{
					ret = ret_end = curr;
				}else{
					/* create a secondary linked list within the main one */
					ret_end->lookup_next = curr;
					ret_end = curr;
				}
			}
		}else{
			if(c->id == curr->c->id && c->type != CONN_LOCAL)
			{
				if(user != NULL && curr->user != NULL && strcmp(user,curr->user) == 0)
					return curr;
				if(ret == NULL)
				{
					ret = ret_end = curr;
				}else{
					ret_end->lookup_next = curr;
					ret_end = curr;
				}
			}
		}
		curr = curr->next;
	}
	return ret;
}

void session_print()
{
	session *curr = etc.sessions;

	while(curr != NULL)
	{
		printf("SESSION socket [%d]\n",curr->c->id);
		printf("SESSION socket type [%d]\n",curr->c->type);
		printf("SESSION socket name [%s]",curr->c->name);
		printf("SESSION user [%s]",curr->user);
		printf("SESSION nick [%s]",curr->nick);
		printf("SESSION status_say [%s]",curr->status_say);
		printf("SESSION status_icon [%s]",curr->status_icon);
		printf("SESSION status_ext [%s]",curr->status_ext);
		printf("SESSION status_flag [%d]\n",curr->status_flag);
		printf("SESSION status_priority [%d]\n",curr->status_priority);
		printf("SESSION activity [%s]",ctime(curr->activity));
		printf("SESSION start [%s]",ctime(curr->start));
		curr = curr->next;
	}
}

void free_session(session *s)
{
	free(s->user);
	free(s->nick);
	free(s->status_say);
	free(s->status_icon);
	free(s->status_ext);
	free(s->activity);
	free(s->start);
	free(s);
}


