#include "jabber.h"

void transport_packet(conn *c, xpt *x)
{
	xpt *xml, *next;
	char *val = NULL;
	
	/* stringpieces->string->xpt to make sure we get xpt in the end */
	next = x->children;
	while(next != NULL)
	{
		if(next->type == XPT_CDATA)
			val = strgrow(val,next->value,1,0);
		next = next->next;
	}
	DBG_VAL("transport_packet",val);

	xml = xpt_str(val);
	free(val);

	if(NULL == xml)
		return;

	if(strcmp(x->name,"r") == 0)
	{
		DBG_MSG("Handling remote packet from transport");
		handle_packet(c, xml);
		free_xpt(xml);
		return;
	}

	if(strcmp(x->name,"error") == 0)
	{
		DBG_MSG("Handling bounced packet from transport");
		if(xml->name != NULL && strcmp(xml->name,"message") == 0)
		{
			transport_errormessage(x, xml);
			handle_packet(c,xml);
		}
		free_xpt(xml);
		return;
	}

	if(strcmp(x->name,"config") == 0)
	{
		DBG_MSG("Handling config packet from transport");
		config_handle(xml->children);
		/* don't free the config, is referenced throughout apps lifetime */
		return;
	}

	syslog_debug(SYSLOG_ERROR, "etherx", "unknown_packet", x->name);

	return;
}


void transport_deliver(char *from_user, char *from_nick, char *to_user, char *to_nick, xpt *x)
{
	xpt *to;
	char *str, *packet, *server;
	jpair *attributes = NULL;

	server = strstr(to_user, "@");
	if(server == NULL)
		return;
	else
		++server;

	from_user = strgrow(NULL,from_user,0,0);
	from_user = strgrow(from_user,"@",1,0);
	from_user = strgrow(from_user,jpair_getval(etc.vars,"name"),1,0);
	x->attributes = jpair_add(x->attributes, "from", from_user);
	x->attributes = jpair_add(x->attributes, "nick", from_nick);

	to = xpt_new_tag(x, "to", NULL);
	if(to_nick != NULL)
		to->attributes = jpair_new(to->attributes, "nick", to_nick, 0);
	xpt_new_cdata(to, to_user);

	attributes = jpair_new(attributes, "to", server, 0);
	attributes = jpair_new(attributes, "from", jpair_getval(etc.vars, "name"), 0);
	str = xpt2str(x);
	packet = create_packet("r", attributes, str);

	etc.transport->packets = packet_add(etc.transport->packets, packet, from_user);

	free_jpair(attributes);
	free(str);
	free(packet);
}

char *create_packet(char *name, jpair *atts, char *body)
{
	xpt *next;
	char *ret = NULL;
	char *buf = NULL;

	ret = strgrow(ret,"<",1,0);
	ret = strgrow(ret,name,1,0);
	while(atts != NULL)
	{
		ret = strgrow(ret," ",1,0);
		ret = strgrow(ret,atts->key,1,0);
		ret = strgrow(ret,"='",1,0);
		ret = strgrow(ret,strescape(atts->val),1,1);
		ret = strgrow(ret,"'",1,0);
		atts = atts->next;
	}
	ret = strgrow(ret,"><![CDATA[",1,0);
	ret = strgrow(ret,body,1,0);
	ret = strgrow(ret,"]]></",1,0);
	ret = strgrow(ret,name,1,0);
	ret = strgrow(ret,">",1,0);

	return ret;
}

/* this is a SAD SAD kludge
 * I'm not sure how to do it better...
 * Anyway, it runs throug the bounced message blurring it into an error message
 * and is then delivered normally
 */
void transport_errormessage(xpt *x, xpt *xml)
{
	xpt *next, *new;
	char *say;
	jpair *par;

	DBG_MSG("transport error message");

	next = xml->children;
	while(next != NULL)
	{
		if(next->type == XPT_TAG && strcmp(next->name, "subject") == 0)
		{
			/* this is a sad kludge to delete the current tag */
			new = xpt_new_tag(NULL, "kludge", NULL);
			new->children = next->children;
			free_xpt(new);
			next->children = NULL;
			xpt_new_cdata(next,"Message Delivery Failed");
		}
		if(next->type == XPT_TAG && strcmp(next->name, "say") == 0)
		{
			say = strgrow(NULL,"Your message to the server '",0,0);
			say = strgrow(say,jpair_getval(x->attributes,"to"),1,0);
			say = strgrow(say,"' failed with the error '",1,0);
			say = strgrow(say,jpair_getval(x->attributes,"error_type"),1,0);
			say = strgrow(say,"'.\nOriginal message:\n",1,0);
			say = strgrow(say,xpt2str_chain(next->children),1,1);
			new = xpt_new_tag(NULL, "kludge", NULL);
			new->children = next->children;
			free_xpt(new);
			next->children = NULL;
			xpt_new_cdata(next,say);
			free(say);
		}
		if(next->type == XPT_TAG && strcmp(next->name, "to") == 0)
		{
			free_jpair(next->attributes);
			next->attributes = NULL;
			new = xpt_new_tag(NULL, "kludge", NULL);
			new->children = next->children;
			free_xpt(new);
			next->children = NULL;
			next->attributes = jpair_new(next->attributes, "nick", jpair_getval(xml->attributes,"nick"), 0);
			xpt_new_cdata(next,jpair_getval(xml->attributes, "from"));
		}
		next = next->next;
	}
	free_jpair(xml->attributes);
	xml->attributes = NULL;
	xml->attributes = jpair_new(xml->attributes,"from","system",0);
	xml->attributes = jpair_new(xml->attributes,"type","error",0);
}
 

