#include "jabber.h"

void sig_die()
{
	ERROR("main","Exiting Jabber Transport");
	longjmp(_jabberlib_ioenv,1);
}

char *make_tag_quick(char *name, char *value)
{
	char *ret;

	if(value == NULL)
		return NULL;

	if(*value == '\0')
		return NULL;

	value = strescape(value);
	ret = malloc(strlen(name)*2 + strlen(value) + 6); 
	sprintf(ret,"<%s>%s</%s>",name,value,name);
	free(value);

	return ret;
}

jpair *get_from(conn *c, xpt *x)
{
	jpair *ret = NULL;
	char *user, *nick, *group;
	session *s;

	if(x == NULL || c == NULL)
		return NULL;

	switch(c->type)
	{
		case CONN_LOCAL:
			user = jpair_getval(x->attributes, "from");
			nick = jpair_getval(x->attributes, "nick");
			if(user == NULL)
				return NULL;
			ret = jpair_new(NULL, user, nick, 0);
			break;

		case CONN_MULTI:
			user = jpair_getval(x->attributes, "from");
			nick = jpair_getval(x->attributes, "nick");
			if(user == NULL)
				return NULL;
			s = session_lookup(c, user, nick);
			if(s == NULL) /* Uhoh! bad client trying to ID a packet as someone else? */
				return NULL;
			ret = jpair_new(NULL, user, nick, 0);
			break;

		case CONN_NORMAL:
			s = session_lookup(c, NULL, NULL);
			if(s == NULL) /* Anonymous user! */
			{
				group = jpair_getval(etc.vars, "anonymous");
				if(group == NULL) /* anonymous users disallowed */
				{
					c->packets = packet_add(c->packets, "<message type=\"error\"><subject>Anonymous Access Denied</subject><say>This server is configured to not allow Anonymous connections.  Please use a valid login to gain access.</say><ext>AnonymousDenied</ext></message>",NULL);
					return NULL;
				}
				s = session_add(c, "anonymous", c->name, group);
			}
			ret = jpair_new(NULL, s->user, s->nick, 0);
			break;
	}

	return ret;
}

jpair *get_to(conn *c, xpt *x)
{
	xpt *next;
	jpair *ret = NULL;
	char *user, *nick;

	if(x == NULL || c == NULL)
		return NULL;

	next = x->children;
	while(next != NULL)
	{
		if(next->type == XPT_TAG && strcmp(next->name, "to") == 0)
		{
			user = xpt2str_chain(next->children);
			nick = jpair_getvalnew(next->attributes, "nick");
			ret = jpair_add(ret, user, nick);
		}
		next = next->next;
	}
	return ret;
}


void local_deliver(char *from, char *user, char *nick, conn *c, xpt *x)
{
	char *str;

	DBG_MSG("Delivering Locally");

	if(c->type == CONN_MULTI)
	{
		x->attributes = jpair_new(x->attributes, "for", user, 0);
		x->attributes = jpair_new(x->attributes, "nick", nick, 0);
	}

	str = xpt2str(x);
	c->packets = packet_add(c->packets, str, from);
	free(str);
}


int islocal(char *address)
{
	int local = FALSE;
	jpair *par;

	if(address == NULL)
		return TRUE;

	par = etc.vars;
	while(par != NULL)
	{
		if(	par->key != NULL &&
			(strcmp(par->key, "alias") == 0 || strcmp(par->key, "name") == 0) &&
			par->val != NULL &&
			strcmp(par->val,address) == 0 )
		{
			/* one of the names for this transport matched the server address, deliver here */
			local = TRUE;
		}
		par = par->next;
	}
	return local;
}


void logger(int level, char *from, char *message)
{
	static time_t t;
	static int log, checklog;
	char *tstr, *tmpctime;
	int check;

	/* initialize and open the logfile if configured to do so */
	if(checklog != 1)
	{
		log = -1;
		if(jpair_get(etc.vars, "errorlog") != NULL)
		{
			checklog = 1;
			log = open(jpair_getval(etc.vars, "errorlog"), O_WRONLY | O_CREAT | O_APPEND);
			if(log < 0)
				ERROR("main", "Unable to open errorlog");
		}
	}

	/* log only at the level told to do so */
	if(level >= etc.debug)
	{
		time(&t);
		tstr = strgrow(NULL,"[",0,0);
		tmpctime = strdup(ctime(&t));
		tmpctime[strlen(tmpctime) - 1] = '\0';
		tstr = strgrow(tstr,tmpctime,1,1);
		tstr = strgrow(tstr,"] ",1,0);
		tstr = strgrow(tstr,from,1,0);
		tstr = strgrow(tstr," \"",1,0);
		if(message != NULL)
			tstr = strgrow(tstr,message,1,0);
		tstr = strgrow(tstr,"\"\n",1,0);

		/* if a logfile is configured and opened, put logs there unless were debugging */
		if(log >= 0 && etc.debug > SYSLOG_DEBUG)
		{
			check = write(log, tstr, strlen(tstr));
			if(check != strlen(tstr))
			{
				log = -1;
				ERROR("main", "Writing to error log file failed!");
				syslog(level, from, message);
			}
		}else{
			printf("%s",tstr);
		}

		free(tstr);
	}
}

